/* Test xdr_bytes, xdr_string behavior on deserialization failure.
   Copyright (C) 2017 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, see
   <http://www.gnu.org/licenses/>.  */

#include <mcheck.h>
#include <rpc/rpc.h>
#include <support/check.h>
#include <support/support.h>

static int
do_test (void)
{
  mtrace ();

  /* If do_own_buffer, allocate the buffer and pass it to the
     deserialization routine.  Otherwise the routine is requested to
     allocate the buffer.  */
  for (int do_own_buffer = 0; do_own_buffer < 2; ++do_own_buffer)
    {
      /* Length 16 MiB, but only 2 bytes of data in the packet.  */
      unsigned char buf[] = "\x01\x00\x00\x00\xff";
      XDR xdrs;
      char *result;
      unsigned int result_len;

      /* Test xdr_bytes.  */
      xdrmem_create (&xdrs, (char *) buf, sizeof (buf), XDR_DECODE);
      result_len = 0;
      if (do_own_buffer)
        {
          char *own_buffer = xmalloc (10);
          result = own_buffer;
          TEST_VERIFY (!xdr_bytes (&xdrs, &result, &result_len, 10));
          TEST_VERIFY (result == own_buffer);
          free (own_buffer);
        }
      else
        {
          result = NULL;
          TEST_VERIFY (!xdr_bytes (&xdrs, &result, &result_len, -1));
          TEST_VERIFY (result == NULL);
        }
      TEST_VERIFY (result_len == 16 * 1024 * 1024);
      xdr_destroy (&xdrs);

      /* Test xdr_string.  */
      xdrmem_create (&xdrs, (char *) buf, sizeof (buf), XDR_DECODE);
      if (do_own_buffer)
        {
          char *own_buffer = xmalloc (10);
          result = own_buffer;
          TEST_VERIFY (!xdr_string (&xdrs, &result, 10));
          TEST_VERIFY (result == own_buffer);
          free (own_buffer);
        }
      else
        {
          result = NULL;
          TEST_VERIFY (!xdr_string (&xdrs, &result, -1));
          TEST_VERIFY (result == NULL);
        }
      xdr_destroy (&xdrs);
    }

  return 0;
}

#include <support/test-driver.c>

